@@ -25,6 +25,7 @@ from wolfcrypt._ffi import lib as _lib
 from wolfcrypt.utils import t2b
 from wolfcrypt.random import Random
 from wolfcrypt.asn import pem_to_der
+from wolfcrypt.hashes import hash_type_to_cls
 
 from wolfcrypt.exceptions import WolfCryptError
 
@@ -272,7 +273,7 @@ if _lib.AES_ENABLED:
             else:
                 raise ValueError("Invalid mode associated to cipher")
 
-if _lib.AESGCM_STREAM:
+if _lib.AESGCM_STREAM_ENABLED:
     class AesGcmStream(object):
         """
         AES GCM Stream
@@ -521,9 +522,9 @@ if _lib.RSA_ENABLED:
 
         if _lib.ASN_ENABLED:
             @classmethod
-            def from_pem(cls, file):
+            def from_pem(cls, file, hash_type=None):
                 der = pem_to_der(file, _lib.PUBLICKEY_TYPE)
-                return cls(der)
+                return cls(key=der, hash_type=hash_type)
 
         def encrypt(self, plaintext):
             """
@@ -597,6 +598,14 @@ if _lib.RSA_ENABLED:
 
                 Returns a string containing the plaintext.
                 """
+                if not self._hash_type:
+                    raise WolfCryptError(("Hash type not set. Cannot verify a "
+                        "PSS signature without a hash type."))
+
+                hash_cls = hash_type_to_cls(self._hash_type)
+                if not hash_cls:
+                    raise WolfCryptError("Unsupported PSS hash type.")
+
                 plaintext = t2b(plaintext)
                 signature = t2b(signature)
                 if self._mgf is None:
@@ -610,7 +619,9 @@ if _lib.RSA_ENABLED:
 
                 if ret < 0:  # pragma: no cover
                     raise WolfCryptError("Verify error (%d)" % ret)
-                ret = _lib.wc_RsaPSS_CheckPadding(plaintext, len(plaintext),
+
+                digest = hash_cls.new(plaintext).digest()
+                ret = _lib.wc_RsaPSS_CheckPadding(digest, len(digest),
                                                   verify, ret, self._hash_type)
 
                 return ret
@@ -620,11 +631,11 @@ if _lib.RSA_ENABLED:
     class RsaPrivate(RsaPublic):
         if _lib.KEYGEN_ENABLED:
             @classmethod
-            def make_key(cls, size, rng=Random()):
+            def make_key(cls, size, rng=Random(), hash_type=None):
                 """
                 Generates a new key pair of desired length **size**.
                 """
-                rsa = cls(None)
+                rsa = cls(hash_type=hash_type)
                 if rsa == None:  # pragma: no cover
                     raise WolfCryptError("Invalid key error (%d)" % ret)
 
@@ -670,9 +681,9 @@ if _lib.RSA_ENABLED:
 
         if _lib.ASN_ENABLED:
             @classmethod
-            def from_pem(cls, file):
+            def from_pem(cls, file, hash_type=None):
                 der = pem_to_der(file, _lib.PRIVATEKEY_TYPE)
-                return cls(der)
+                return cls(key=der, hash_type=hash_type)
 
         if _lib.KEYGEN_ENABLED:
             def encode_key(self):
@@ -776,11 +787,22 @@ if _lib.RSA_ENABLED:
 
                 Returns a string containing the signature.
                 """
+                if not self._hash_type:
+                    raise WolfCryptError(("Hash type not set. Cannot verify a "
+                        "PSS signature without a hash type."))
+
+                hash_cls = hash_type_to_cls(self._hash_type)
+                if not hash_cls:
+                    raise WolfCryptError("Unsupported PSS hash type.")
+
                 plaintext = t2b(plaintext)
+                digest = hash_cls.new(plaintext).digest()
+
                 signature = _ffi.new("byte[%d]" % self.output_size)
                 if self._mgf is None:
                     self._get_mgf()
-                ret = _lib.wc_RsaPSS_Sign(plaintext, len(plaintext),
+
+                ret = _lib.wc_RsaPSS_Sign(digest, len(digest),
                                           signature, self.output_size,
                                           self._hash_type, self._mgf,
                                           self.native_object,
@@ -981,7 +1003,8 @@ if _lib.ECC_ENABLED:
             if ret < 0:
                 raise WolfCryptError("Key generation error (%d)" % ret)
 
-            if _lib.ECC_TIMING_RESISTANCE_ENABLED:
+            if _lib.ECC_TIMING_RESISTANCE_ENABLED and (not _lib.FIPS_ENABLED or
+               _lib.FIPS_VERSION > 2):
                 ret = _lib.wc_ecc_set_rng(ecc.native_object, rng.native_object)
                 if ret < 0:
                     raise WolfCryptError("Error setting ECC RNG (%d)" % ret)
